% Copyright 2018. All Rights Reserved
% Code by Shuang Li
% For Paper, "Atomic Norm Minimization for Modal Analysis from Random and Compressed Samples"
% IEEE transactions on signal processing, 2018
% by Shuang Li, Dehui Yang, Gongguo Tang, and Michael B. Wakin
% This code is used to reproduce Figure 4(a,b).

%%
clc;clear;close all;

trial=100;    % number of trials
Nshape=3;     % number of active modes
Nsample=80;   % number of uniform samples per sensor
Ts=1;
Time=0:Ts:Ts*(Nsample-1);    % sampling interval
%M=30;
%nsensor=[2 10];
M=[3 5 7 9 11 15 20 25 30];  % number of compressed measurements per sensor
nsensor=[2 4 8 16];          % number of sensor
c=ones(Nshape,size(M,2))-0.01;   % We view recovery as a success if MAC >= c
countj=zeros(size(nsensor,2),size(M,2));
counts=zeros(size(nsensor,2),size(M,2));
XXj=zeros(Nsample,max(nsensor),size(M,2),size(nsensor,2),trial);
XXs=zeros(Nsample,max(nsensor),size(M,2),size(nsensor,2),trial);
msmsj=zeros(max(nsensor),Nshape,size(M,2),size(nsensor,2),trial);
msmss=zeros(max(nsensor),Nshape,size(M,2),size(nsensor,2),trial);
ffj=zeros(Nshape,size(M,2),size(nsensor,2),trial);
ffs=zeros(Nshape,size(M,2),size(nsensor,2),trial);
OriginalData=zeros(Nsample,max(nsensor),trial);
Originalms=zeros(max(nsensor),Nshape,trial);
for m=1:trial
    Nsensor=max(nsensor);
    mshape=randn(Nsensor,Nshape);    % generate random mode shapes
    if Nsensor > 1
        mshape = mshape*diag(1./norms(mshape));
    else
        mshape = ones(1,Nshape);
    end
    Originalms(:,:,m)=mshape;  % save the true mode shapes
    
    A=[1 2 3];                 % amplitudes
    freq=[0.1; 0.15; 0.5];     % true frequencies.
    Q=exp(1j*2*pi*freq*Time);
    
    Data=mshape*diag(A)*Q;     
    Data= transpose(Data);     % data matrix
    OriginalData(:,:,m)=Data;  % save the true data matrix
    
    for n=1:size(nsensor,2)
        MAC=zeros(Nshape,size(M,2));
        SMAC=zeros(Nshape,size(M,2));
        B=zeros(nsensor(n),Nshape);
        SB=zeros(nsensor(n),Nshape);
        for i=1:size(M,2)
            Nsensor=nsensor(n);
            Phi=zeros(M(i)*Nsensor,Nsample*Nsensor); % measurement matrix
            for l=1:Nsensor
                Phi((l-1)*M(i)+1:l*M(i),(l-1)*Nsample+1:l*Nsample)=randn(M(i),Nsample);
            end
            Vyy=Phi*vec(Data(:,1:Nsensor));    % compressed measurements
            %==================Joint recovery================================
            cvx_begin quiet
            variable u0;
            variable u(Nsample-1) complex;
            variable Tu(Nsample,Nsample) hermitian;
            variable X(Nsample,Nsensor) complex;
            variable W(Nsensor,Nsensor) Hermitian;
            dual variable Y0
            minimize(0.5/Nsample*trace(Tu)+0.5*trace(W))
            subject to
            Tu == toeplitz([u0;conj(u)],[u0;conj(u)]');
            Y0:[Tu   X; ...
                X'   W] == semidefinite(Nsample+Nsensor,Nsample+Nsensor);
            Vyy == Phi*vec(X);   
            cvx_end
            
            XXj(:,1:Nsensor,i,n,m)=X;    % save the recovered data matrix
            
            Y=-2*Y0(1:Nsample,Nsample+1:Nsample+Nsensor);   % dual solution
            
            f=0:0.001:1;
            a=exp(1j*2*pi*Time'*f);   % atoms
            
            Q=Y'*a;                   % dual polynomial
            if Nsensor > 1
                NormQ=norms(Q);
            else
                NormQ=abs(Q);
            end
            
            [pks,locs] = findpeaks(NormQ,'minpeakdistance',1,'MinPeakHeight',0.99);
            if size(locs,2) < Nshape
                Freq_est(1:size(locs,2))=f(locs);
                ffj(1:size(locs,2),i,n,m)=f(locs); % save the recovered frequencies
                B(:,1:size(locs,2))=Q(:,locs);
                msmsj(1:Nsensor,1:size(locs,2),i,n,m)=Q(:,locs); % save the recovered mode shapes
            else
                Freq_est=f(locs(1:3));
                ffj(:,i,n,m)=f(locs(1:3)); % save the recovered frequencies
                B=Q(:,locs(1:3));
                msmsj(1:Nsensor,:,i,n,m)=Q(:,locs(1:3));  % save the recovered mode shapes
            end
            
            ms=mshape(1:Nsensor,:);
            ms = ms*diag(1./norms(ms));
            MAC(:,i)=diag(abs(ms'*B));
            
            %==============Seperate recovery================================
            
            L=Nsensor;
            Nsensor=1;
            Sfreq_est=zeros(L,Nshape);
            SB=zeros(L,Nshape);
            for j=1:L
                fprintf('Trial=%d,Nsensor=%d,M=%d,L=%d\n',m,nsensor(n),M(i),j)
                cvx_begin quiet
                variable u0;
                variable u(Nsample-1) complex;
                variable Tu(Nsample,Nsample) hermitian;
                variable X(Nsample,Nsensor) complex;
                variable W(Nsensor,Nsensor) Hermitian;
                dual variable Y0
                minimize(0.5/Nsample*trace(Tu)+0.5*trace(W))
                subject to
                Tu == toeplitz([u0;conj(u)],[u0;conj(u)]');
                Y0:[Tu   X; ...
                    X'   W] == semidefinite(Nsample+Nsensor,Nsample+Nsensor);
                Vyy((j-1)*M(i)+1:j*M(i))==Phi((j-1)*M(i)+1:j*M(i),(j-1)*Nsample+1:j*Nsample)*X;
                cvx_end
                
                XXs(:,j,i,n,m)=X;    % save the recovered data matrix
                
                Y=-2*Y0(1:Nsample,Nsample+1:Nsample+Nsensor);   % dual solution
                
                f=0:0.001:1;
                a=exp(1j*2*pi*Time'*f);     % atoms
                
                Q=Y'*a;                     % dual solution
                if Nsensor > 1
                    NormQ=norms(Q);
                else
                    NormQ=abs(Q);
                end
                
                [pks,locs] = findpeaks(NormQ,'minpeakdistance',1,'MinPeakHeight',0.99);
                
                if size(locs,2) < Nshape
                    Sfreq_est(j,1:size(locs,2))=f(locs);
                else
                    Sfreq_est(j,:)=f(locs(1:3));
                end
            end
            f_est=mean(Sfreq_est);      % estimated frequencies
            ffs(:,i,n,m)=f_est;         % save the recovered frequencies
            F=exp(1j*2*pi*Time'*f_est);
            C=pinv(F)*XXs(:,1:nsensor(n),i,n,m);
            C=transpose(C);
            C= C*diag(1./norms(C));
            msmss(1:nsensor(n),:,i,n,m)=C;   % save the recovered mode shapes
            
            ms=Originalms(1:nsensor(n),:,m);
            ms = ms*diag(1./norms(ms));
            SMAC(:,i)=diag(abs(ms'*C));
            
            
        end
        
        indx=(sum(MAC>=c)==3);      % successful recovery
        countj(n,indx)=countj(n,indx)+1
        indx=(sum(SMAC>=c)==3);      % successful recovery
        counts(n,indx)=counts(n,indx)+1
    end
end
pj=countj./trial;
ps=counts./trial;

%% Show Results

% Data
NN=size(XXj,4);
MM=size(XXj,3);
Nshape=size(Originalms,2);
err0=1e-5;
countXj=zeros(NN,MM);
countXs=zeros(NN,MM);
for t=1:trial
    X=OriginalData(:,:,t);
    for n=1:NN
        Nsensor=nsensor(n);
        for m=1:MM
            errXj=norm(XXj(:,1:Nsensor,m,n,t)-OriginalData(:,1:Nsensor,t),'fro')/norm(OriginalData(:,1:Nsensor,t),'fro');
            errXs=norm(XXs(:,1:Nsensor,m,n,t)-OriginalData(:,1:Nsensor,t),'fro')/norm(OriginalData(:,1:Nsensor,t),'fro');
            if errXj<=err0
                countXj(n,m)=countXj(n,m)+1;
            end
            if errXs<=err0
                countXs(n,m)=countXs(n,m)+1;
            end
        end
    end
end
pXj=countXj./trial;
pXs=countXs./trial;

figure
lw=6;
fs=29;
plot(M,pXj(1,:),'b-','Linewidth',lw);hold on
plot(M,pXs(1,:),'b--','Linewidth',lw);hold on
plot(M,pXj(2,:),'g-','Linewidth',lw);hold on
plot(M,pXs(2,:),'g--','Linewidth',lw);hold on
plot(M,pXj(3,:),'k-','Linewidth',lw);hold on
plot(M,pXs(3,:),'k--','Linewidth',lw);hold on
plot(M,pXj(4,:),'m-','Linewidth',lw);hold on
plot(M,pXs(4,:),'m--','Linewidth',lw);hold on
axis('tight')
xlabel('Number of measurements per sensor','Interpreter','LaTex','FontSize',fs)
ylabel('Probability of successful recovery (Data)','FontSize',fs)
text([7.2; 9.0; 9.4; 10.4],[0.93; 0.86; 0.75; 0.55],{'16';'8';'4';'2'},'FontSize',fs)
text([11.6; 12.6; 13.3; 13.6],[0.30; 0.2; 0.12; 0.05],{'2';'4';'8';'16'},'FontSize',fs)
set(gca,'FontSize',fs);

% Mode shapes
c=ones(Nshape,MM)-0.01;
countmsj=zeros(NN,MM);
countmss=zeros(NN,MM);
for t=1:trial
    X=OriginalData(:,:,t);
    for n=1:NN
        
        MAC=zeros(Nshape,MM);
        SMAC=zeros(Nshape,MM);
        for m=1:MM
            Nsensor=nsensor(n);
            ms=Originalms(1:Nsensor,:,t);
            ms = ms*diag(1./norms(ms));
            B=msmsj(1:Nsensor,:,m,n,t);
            MAC(:,m)=diag(abs(ms'*B));
            SB=msmss(1:Nsensor,:,m,n,t);
            SMAC(:,m)=diag(abs(ms'*SB));
        end
        indx=(sum(MAC>=c)==3);      % successful recovery
        countmsj(n,indx)=countmsj(n,indx)+1;
        indx=(sum(SMAC>=c)==3);      % successful recovery
        countmss(n,indx)=countmss(n,indx)+1;
    end
end
pmsj=countmsj./trial;
pmss=countmss./trial;

figure
lw=6;
fs=29;
plot(M,pmsj(1,:),'b-','Linewidth',lw);hold on
plot(M,pmss(1,:),'b--','Linewidth',lw);hold on
plot(M,pmsj(2,:),'g-','Linewidth',lw);hold on
plot(M,pmss(2,:),'g--','Linewidth',lw);hold on
plot(M,pmsj(3,:),'k-','Linewidth',lw);hold on
plot(M,pmss(3,:),'k--','Linewidth',lw);hold on
plot(M,pmsj(4,:),'m-','Linewidth',lw);hold on
plot(M,pmss(4,:),'m--','Linewidth',lw);hold on
axis('tight')
xlabel('Number of measurements per sensor','Interpreter','LaTex','FontSize',fs)
ylabel('Probability of successful recovery (ModeShapes)','FontSize',fs)
text([7.0; 8.7; 9.8; 10],[0.8; 0.68; 0.58; 0.47],{'16';'8';'4';'2'},'FontSize',fs)
text([12.1; 12.9; 13.6; 14.3],[0.31; 0.21; 0.12; 0.04],{'2';'4';'8';'16'},'FontSize',fs)
set(gca,'FontSize',fs);
