% Copyright 2018. All Rights Reserved
% Code by Shuang Li
% For Paper, "Atomic Norm Minimization for Modal Analysis from Random and Compressed Samples"
% IEEE transactions on signal processing, 2018
% by Shuang Li, Dehui Yang, Gongguo Tang, and Michael B. Wakin
% This code is used to reproduce Figure 6.

%%
clc;clear; close all;

Nsensor=10;                % number of sensors
Nshape=3;                  % number of active modes
Nsample=80;                % number of uniform samples 
Ts=1;            
Time=0:Ts:Ts*(Nsample-1);

A=[1 2 3];                 % amplitudes
freq=[0.1; 0.15; 0.5 ];    % true frequencies.
QQ=exp(1j*2*pi*freq*Time);
SNR=15:5:50;               % signal to noise ratio
Trial=50;                  % number of trials
MSE=zeros(Trial,length(SNR));
RSE=zeros(Trial,length(SNR));
Sigma2=zeros(Trial,length(SNR));
for t=1:Trial
    mshape=randn(Nsensor,Nshape);     % generate random mode shapes
    if Nsensor > 1
        mshape = mshape*diag(1./norms(mshape));
    else 
        mshape = ones(1,Nshape);
    end

    Data=mshape*diag(A)*QQ; 
    Data= transpose(Data);            % data matrix
    [D1, D2]=size(Data);    
    ED = norm(Data,'fro')/sqrt(D2);
    sigma=ED*10.^(-0.05*SNR)/sqrt(D1);
    Sigma2(t,:)=sigma.^2;             % noise variance
    for n=1:length(SNR)
        fprintf('Trial=%d,SNR=%d\n',t,SNR(n))
        ssigma=sigma(n);
        Z = Data+ssigma/sqrt(2)*randn(D1,D2)+1j*ssigma/sqrt(2)*randn(D1,D2);
        tau=1*ssigma*sqrt(4*Nsample*Nsensor*log(Nsample));

        [XX,~,~] = atomic_admm(Z,tau); % XX is the de-noising data
        MSE(t,n)=norm(XX-Data,'fro')^2;  
        RSE(t,n)=norm(XX-Data,'fro')/norm(Data,'fro');  
    end
end
Sigma2=mean(Sigma2);
MSE=mean(MSE);
RSE=mean(RSE);
%% Show results
figure
lw=6;
fs=29;
loglog(Sigma2,MSE,'r-*','linewidth',lw);
xlabel('$\sigma^2$','Interpreter','LaTex','FontSize',fs)    
ylabel('MSE','Interpreter','LaTex','FontSize',fs)
axis tight;
set(gca,'FontSize',fs);

figure
lw=6;
fs=29;
plot(SNR,RSE,'r-*','linewidth',lw);
xlabel('SNR(dB)','Interpreter','LaTex','FontSize',fs)    
ylabel('Relative Error','Interpreter','LaTex','FontSize',fs)
axis tight;
set(gca,'FontSize',fs);



